@@ -47,11 +47,9 @@ typedef struct ProresVulkanDecodePicture {
 } ProresVulkanDecodePicture;
 
 typedef struct ProresVulkanDecodeContext {
-    struct ProresVulkanShaderVariants {
-        FFVulkanShader reset;
-        FFVulkanShader vld;
-        FFVulkanShader idct;
-    } shaders[2]; /* Progressive/interlaced */
+    FFVulkanShader reset;
+    FFVulkanShader vld;
+    FFVulkanShader idct;
 
     AVBufferPool *metadata_pool;
 } ProresVulkanDecodeContext;
@@ -163,7 +161,6 @@ static int vk_prores_end_frame(AVCodecContext *avctx)
 
     ProresVkParameters pd;
     FFVkBuffer *slice_data, *metadata;
-    struct ProresVulkanShaderVariants *shaders;
     VkImageMemoryBarrier2 img_bar[AV_NUM_DATA_POINTERS];
     VkBufferMemoryBarrier2 buf_bar[2];
     int nb_img_bar = 0, nb_buf_bar = 0, err;
@@ -179,8 +176,6 @@ static int vk_prores_end_frame(AVCodecContext *avctx)
     slice_data = (FFVkBuffer *)vp->slices_buf->data;
     metadata   = (FFVkBuffer *)pp->metadata_buf->data;
 
-    shaders = &pv->shaders[pr->frame_type != 0];
-
     pd = (ProresVkParameters) {
         .slice_data       = slice_data->address,
         .bitstream_size   = pp->bitstream_size,
@@ -253,14 +248,14 @@ static int vk_prores_end_frame(AVCodecContext *avctx)
     nb_img_bar = nb_buf_bar = 0;
 
     /* Reset */
-    ff_vk_shader_update_img_array(&ctx->s, exec, &shaders->reset,
+    ff_vk_shader_update_img_array(&ctx->s, exec, &pv->reset,
                                   pr->frame, vp->view.out,
                                   0, 0,
                                   VK_IMAGE_LAYOUT_GENERAL,
                                   VK_NULL_HANDLE);
 
-    ff_vk_exec_bind_shader(&ctx->s, exec, &shaders->reset);
-    ff_vk_shader_update_push_const(&ctx->s, exec, &shaders->reset,
+    ff_vk_exec_bind_shader(&ctx->s, exec, &pv->reset);
+    ff_vk_shader_update_push_const(&ctx->s, exec, &pv->reset,
                                    VK_SHADER_STAGE_COMPUTE_BIT,
                                    0, sizeof(pd), &pd);
 
@@ -284,24 +279,24 @@ static int vk_prores_end_frame(AVCodecContext *avctx)
     nb_img_bar = nb_buf_bar = 0;
 
     /* Entropy decode */
-    ff_vk_shader_update_desc_buffer(&ctx->s, exec, &shaders->vld,
+    ff_vk_shader_update_desc_buffer(&ctx->s, exec, &pv->vld,
                                     0, 0, 0,
                                     metadata, 0,
                                     pp->slice_offsets_sz,
                                     VK_FORMAT_UNDEFINED);
-    ff_vk_shader_update_desc_buffer(&ctx->s, exec, &shaders->vld,
+    ff_vk_shader_update_desc_buffer(&ctx->s, exec, &pv->vld,
                                     0, 1, 0,
                                     metadata, pp->slice_offsets_sz,
                                     pp->mb_params_sz,
                                     VK_FORMAT_UNDEFINED);
-    ff_vk_shader_update_img_array(&ctx->s, exec, &shaders->vld,
+    ff_vk_shader_update_img_array(&ctx->s, exec, &pv->vld,
                                   pr->frame, vp->view.out,
                                   0, 2,
                                   VK_IMAGE_LAYOUT_GENERAL,
                                   VK_NULL_HANDLE);
 
-    ff_vk_exec_bind_shader(&ctx->s, exec, &shaders->vld);
-    ff_vk_shader_update_push_const(&ctx->s, exec, &shaders->vld,
+    ff_vk_exec_bind_shader(&ctx->s, exec, &pv->vld);
+    ff_vk_shader_update_push_const(&ctx->s, exec, &pv->vld,
                                    VK_SHADER_STAGE_COMPUTE_BIT,
                                    0, sizeof(pd), &pd);
 
@@ -341,19 +336,19 @@ static int vk_prores_end_frame(AVCodecContext *avctx)
     nb_img_bar = nb_buf_bar = 0;
 
     /* Inverse transform */
-    ff_vk_shader_update_desc_buffer(&ctx->s, exec, &shaders->idct,
+    ff_vk_shader_update_desc_buffer(&ctx->s, exec, &pv->idct,
                                     0, 0, 0,
                                     metadata, pp->slice_offsets_sz,
                                     pp->mb_params_sz,
                                     VK_FORMAT_UNDEFINED);
-    ff_vk_shader_update_img_array(&ctx->s, exec, &shaders->idct,
+    ff_vk_shader_update_img_array(&ctx->s, exec, &pv->idct,
                                   pr->frame, vp->view.out,
                                   0, 1,
                                   VK_IMAGE_LAYOUT_GENERAL,
                                   VK_NULL_HANDLE);
 
-    ff_vk_exec_bind_shader(&ctx->s, exec, &shaders->idct);
-    ff_vk_shader_update_push_const(&ctx->s, exec, &shaders->idct,
+    ff_vk_exec_bind_shader(&ctx->s, exec, &pv->idct);
+    ff_vk_shader_update_push_const(&ctx->s, exec, &pv->idct,
                                    VK_SHADER_STAGE_COMPUTE_BIT,
                                    0, sizeof(pd), &pd);
 
@@ -439,13 +434,10 @@ fail:
 static void vk_decode_prores_uninit(FFVulkanDecodeShared *ctx)
 {
     ProresVulkanDecodeContext *pv = ctx->sd_ctx;
-    int i;
 
-    for (i = 0; i < FF_ARRAY_ELEMS(pv->shaders); ++i) {
-        ff_vk_shader_free(&ctx->s, &pv->shaders[i].reset);
-        ff_vk_shader_free(&ctx->s, &pv->shaders[i].vld);
-        ff_vk_shader_free(&ctx->s, &pv->shaders[i].idct);
-    }
+    ff_vk_shader_free(&ctx->s, &pv->reset);
+    ff_vk_shader_free(&ctx->s, &pv->vld);
+    ff_vk_shader_free(&ctx->s, &pv->idct);
 
     av_buffer_pool_uninit(&pv->metadata_pool);
 
@@ -456,12 +448,13 @@ static int vk_decode_prores_init(AVCodecContext *avctx)
 {
     FFVulkanDecodeContext *dec = avctx->internal->hwaccel_priv_data;
     FFVulkanDecodeShared  *ctx = NULL;
+    ProresContext          *pr = avctx->priv_data;
 
     AVHWFramesContext *out_frames_ctx;
     ProresVulkanDecodeContext *pv;
     FFVkSPIRVCompiler *spv;
     FFVulkanDescriptorSetBinding *desc_set;
-    int max_num_mbs, i, err;
+    int max_num_mbs, err;
 
     max_num_mbs = (avctx->coded_width >> 4) * (avctx->coded_height >> 4);
 
@@ -486,80 +479,75 @@ static int vk_decode_prores_init(AVCodecContext *avctx)
 
     ctx->sd_ctx_free = vk_decode_prores_uninit;
 
-    for (i = 0; i < FF_ARRAY_ELEMS(pv->shaders); ++i) { /* Progressive/interlaced */
-        struct ProresVulkanShaderVariants *shaders = &pv->shaders[i];
+    desc_set = (FFVulkanDescriptorSetBinding []) {
+        {
+            .name       = "dst",
+            .type       = VK_DESCRIPTOR_TYPE_STORAGE_IMAGE,
+            .dimensions = 2,
+            .mem_layout = ff_vk_shader_rep_fmt(out_frames_ctx->sw_format,
+            FF_VK_REP_NATIVE),
+            .mem_quali  = "writeonly",
+            .elems      = av_pix_fmt_count_planes(out_frames_ctx->sw_format),
+            .stages     = VK_SHADER_STAGE_COMPUTE_BIT,
+        },
+    };
+    RET(init_shader(avctx, &ctx->s, &ctx->exec_pool, spv, &pv->reset,
+                    "prores_dec_reset", "main", desc_set, 1,
+                    ff_source_prores_reset_comp, 0x080801, pr->frame_type != 0));
+    desc_set = (FFVulkanDescriptorSetBinding []) {
+        {
+            .name        = "slice_offsets_buf",
+            .type        = VK_DESCRIPTOR_TYPE_STORAGE_BUFFER,
+            .stages      = VK_SHADER_STAGE_COMPUTE_BIT,
+            .mem_quali   = "readonly",
+            .buf_content = "uint32_t slice_offsets",
+            .buf_elems   = max_num_mbs + 1,
+        },
+        {
+            .name        = "quant_idx_buf",
+            .type        = VK_DESCRIPTOR_TYPE_STORAGE_BUFFER,
+            .stages      = VK_SHADER_STAGE_COMPUTE_BIT,
+            .mem_quali   = "writeonly",
+            .buf_content = "uint8_t quant_idx",
+            .buf_elems   = max_num_mbs,
+        },
+        {
+            .name       = "dst",
+            .type       = VK_DESCRIPTOR_TYPE_STORAGE_IMAGE,
+            .dimensions = 2,
+            .mem_layout = ff_vk_shader_rep_fmt(out_frames_ctx->sw_format,
+            FF_VK_REP_NATIVE),
+            .mem_quali  = "writeonly",
+            .elems      = av_pix_fmt_count_planes(out_frames_ctx->sw_format),
+            .stages     = VK_SHADER_STAGE_COMPUTE_BIT,
+        },
+    };
+    RET(init_shader(avctx, &ctx->s, &ctx->exec_pool, spv, &pv->vld,
+                    "prores_dec_vld", "main", desc_set, 3,
+                    ff_source_prores_vld_comp, 0x080801, pr->frame_type != 0));
 
-        desc_set = (FFVulkanDescriptorSetBinding []) {
-            {
-                .name       = "dst",
-                .type       = VK_DESCRIPTOR_TYPE_STORAGE_IMAGE,
-                .dimensions = 2,
-                .mem_layout = ff_vk_shader_rep_fmt(out_frames_ctx->sw_format,
-                                                   FF_VK_REP_NATIVE),
-                .mem_quali  = "writeonly",
-                .elems      = av_pix_fmt_count_planes(out_frames_ctx->sw_format),
-                .stages     = VK_SHADER_STAGE_COMPUTE_BIT,
-            },
-        };
-        RET(init_shader(avctx, &ctx->s, &ctx->exec_pool, spv, &shaders->reset,
-                        "prores_dec_reset", "main", desc_set, 1,
-                        ff_source_prores_reset_comp, 0x080801, i));
-
-        desc_set = (FFVulkanDescriptorSetBinding []) {
-            {
-                .name        = "slice_offsets_buf",
-                .type        = VK_DESCRIPTOR_TYPE_STORAGE_BUFFER,
-                .stages      = VK_SHADER_STAGE_COMPUTE_BIT,
-                .mem_quali   = "readonly",
-                .buf_content = "uint32_t slice_offsets",
-                .buf_elems   = max_num_mbs + 1,
-            },
-            {
-                .name        = "quant_idx_buf",
-                .type        = VK_DESCRIPTOR_TYPE_STORAGE_BUFFER,
-                .stages      = VK_SHADER_STAGE_COMPUTE_BIT,
-                .mem_quali   = "writeonly",
-                .buf_content = "uint8_t quant_idx",
-                .buf_elems   = max_num_mbs,
-            },
-            {
-                .name       = "dst",
-                .type       = VK_DESCRIPTOR_TYPE_STORAGE_IMAGE,
-                .dimensions = 2,
-                .mem_layout = ff_vk_shader_rep_fmt(out_frames_ctx->sw_format,
-                                                   FF_VK_REP_NATIVE),
-                .mem_quali  = "writeonly",
-                .elems      = av_pix_fmt_count_planes(out_frames_ctx->sw_format),
-                .stages     = VK_SHADER_STAGE_COMPUTE_BIT,
-            },
-        };
-        RET(init_shader(avctx, &ctx->s, &ctx->exec_pool, spv, &shaders->vld,
-                        "prores_dec_vld", "main", desc_set, 3,
-                        ff_source_prores_vld_comp, 0x080801, i));
-
-        desc_set = (FFVulkanDescriptorSetBinding []) {
-            {
-                .name        = "quant_idx_buf",
-                .type        = VK_DESCRIPTOR_TYPE_STORAGE_BUFFER,
-                .stages      = VK_SHADER_STAGE_COMPUTE_BIT,
-                .mem_quali   = "readonly",
-                .buf_content = "uint8_t quant_idx",
-                .buf_elems   = max_num_mbs,
-            },
-            {
-                .name       = "dst",
-                .type       = VK_DESCRIPTOR_TYPE_STORAGE_IMAGE,
-                .dimensions = 2,
-                .mem_layout = ff_vk_shader_rep_fmt(out_frames_ctx->sw_format,
-                                                   FF_VK_REP_NATIVE),
-                .elems      = av_pix_fmt_count_planes(out_frames_ctx->sw_format),
-                .stages     = VK_SHADER_STAGE_COMPUTE_BIT,
-            },
-        };
-        RET(init_shader(avctx, &ctx->s, &ctx->exec_pool, spv, &shaders->idct,
-                        "prores_dec_idct", "main", desc_set, 2,
-                        ff_source_prores_idct_comp, 0x200201, i));
-    }
+    desc_set = (FFVulkanDescriptorSetBinding []) {
+        {
+            .name        = "quant_idx_buf",
+            .type        = VK_DESCRIPTOR_TYPE_STORAGE_BUFFER,
+            .stages      = VK_SHADER_STAGE_COMPUTE_BIT,
+            .mem_quali   = "readonly",
+            .buf_content = "uint8_t quant_idx",
+            .buf_elems   = max_num_mbs,
+        },
+        {
+            .name       = "dst",
+            .type       = VK_DESCRIPTOR_TYPE_STORAGE_IMAGE,
+            .dimensions = 2,
+            .mem_layout = ff_vk_shader_rep_fmt(out_frames_ctx->sw_format,
+            FF_VK_REP_NATIVE),
+            .elems      = av_pix_fmt_count_planes(out_frames_ctx->sw_format),
+            .stages     = VK_SHADER_STAGE_COMPUTE_BIT,
+        },
+    };
+    RET(init_shader(avctx, &ctx->s, &ctx->exec_pool, spv, &pv->idct,
+                    "prores_dec_idct", "main", desc_set, 2,
+                    ff_source_prores_idct_comp, 0x200201, pr->frame_type != 0));
 
     err = 0;
 
@@ -573,11 +561,9 @@ static void vk_prores_free_frame_priv(AVRefStructOpaque _hwctx, void *data)
 {
     AVHWDeviceContext    *dev_ctx = _hwctx.nc;
     ProresVulkanDecodePicture *pp = data;
-    FFVulkanDecodePicture *vp = &pp->vp;
 
     ff_vk_decode_free_frame(dev_ctx, &pp->vp);
 
-    av_buffer_unref(&vp->slices_buf);
     av_buffer_unref(&pp->metadata_buf);
 }
 
